/*
 * Copyright (c) 1999 Greg Haerr <greg@censoft.com>
 *
 * Nano-X Core Protocol Client Request Handling Routines
 */ 
#include <stdio.h>
#include <unistd.h>
#include <malloc.h>
#include <assert.h>
#include "serv.h"
#include "nxproto.h"

#define SZREQBUF	2048	/* initial request buffer size*/

/* queued request buffer*/
typedef struct {
	BYTE8 *	bufptr;		/* next unused buffer location*/
	BYTE8 *	bufmax;		/* max buffer location*/
	BYTE8 *	buffer;		/* request buffer*/
} REQBUF;

static REQBUF	reqbuf;		/* request buffer*/
extern int sock;

/* Allocate a request buffer of passed size and fill in header fields*/
void *
nxAllocReq(int type, long size, long extra)
{
	nxReq *	req;
	long	aligned_len;

	/* variable size requests must be hand-padded*/
	if(extra)
		assert((size & (long)(ALIGNSZ-1)) == 0);

	/* calculate aligned length of request buffer*/
	aligned_len = (size + extra + (long)(ALIGNSZ-1)) & ~(long)(ALIGNSZ-1);

	/* verify we're not greater than max request size*/
	assert(aligned_len <= MAXREQUESTSZ);

	/* flush buffer if required, and allocate larger one if required*/
	if(reqbuf.bufptr + aligned_len >= reqbuf.bufmax)
		nxFlushReq(aligned_len);

	/* fill in request header*/
	req = (nxReq *)reqbuf.bufptr;
	req->reqType = (BYTE8)type;
	req->hilength = (BYTE8)((size + extra) >> 16);
	req->length = (UINT16)(size + extra);
	reqbuf.bufptr += aligned_len;
	return req;
}

/* Flush request buffer if required, possibly reallocate buffer size*/
void
nxFlushReq(long newsize)
{
	/* handle one-time initialization case*/
	if(reqbuf.buffer == NULL) {
		if(newsize < (long)SZREQBUF)
			newsize = SZREQBUF;
		reqbuf.buffer = malloc(newsize);
		if(!reqbuf.buffer) {
			printf("nxFlushReq: Can't allocate initial request buffer\n");
			exit(1);
		}
		reqbuf.bufptr = reqbuf.buffer;
		reqbuf.bufmax = reqbuf.buffer + newsize;
		return;
	}
	
	/* flush buffer if required*/
	if(reqbuf.bufptr > reqbuf.buffer) {
		char *	buf = reqbuf.buffer;
		int	todo = reqbuf.bufptr - reqbuf.buffer;
		int	written;

/*printf("GrFlush\n");*/
		do {
			written = write(sock, buf, todo);
			if(written <= 0) {
				printf("nxFlushReq: Error on buffer flush\n");
				exit(1);
			}
			buf += written;
			todo -= written;
		} while(todo > 0);

		reqbuf.bufptr = reqbuf.buffer;
	}

	/* allocate larger buffer for current request, if needed*/
	if(reqbuf.bufptr + newsize >= reqbuf.bufmax) {
		reqbuf.buffer = realloc(reqbuf.buffer, newsize);
		if(!reqbuf.buffer) {
			printf("nxFlushReq: Can't reallocate request buffer\n");
			exit(1);
		}
		reqbuf.bufptr = reqbuf.buffer;
		reqbuf.bufmax = reqbuf.buffer + newsize;
	}
}
